/*
 * exempi - xmptree.cpp
 *
 * Copyright (C) 2007 Hubert Figuiere
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
 * USA
 */

#include <map>
#include <string>
#include <iostream>


#include "xmptree.h"
#include "xmpxmlparser.h"

namespace Exempi {

	using Internal::XmpXmlParser;


	XmpSchema::const_iterator XmpSchema::begin() const
	{
		return properties.begin();
	}

	XmpSchema::const_iterator XmpSchema::end() const
	{
		return properties.end();
	}


	XmpSchema::iterator XmpSchema::begin()
	{
		return properties.begin();
	}

	XmpSchema::iterator XmpSchema::end()
	{
		return properties.end();
	}


	struct XmpTree::Private
	{
		XmpTree::DataStore schemas;
	};


	XmpTree::XmpTree()
		: d(new Private())
	{

	}

	XmpTree::~XmpTree()
	{
		delete d;
	}


	bool XmpTree::parse(XmpLoader *loader)
	{
		return loader->parse(d->schemas);
	}

	XmpTree::iterator XmpTree::begin()
	{
		return d->schemas.begin();
	}


	XmpTree::iterator XmpTree::end()
	{
		return d->schemas.end();
	}


}
