/*
 * exempi - xmptree.h
 *
 * Copyright (C) 2007 Hubert Figuiere
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
 * USA
 */


#ifndef __EXEMPI_XMPTREE_H_
#define __EXEMPI_XMPTREE_H_

#include <string>
#include <map>

#include "xmpvalue.h"

namespace Exempi {

	class XmpLoader;

	/** An XMP Schema */
	class XmpSchema 
	{
	public:
		typedef std::map<std::string, XmpValue> Properties;
		typedef Properties::iterator iterator;
		typedef Properties::const_iterator const_iterator;
		typedef Properties::value_type value_type;

		XmpSchema(const char* pfx)
			: prefix(pfx)
			{
			}
		iterator begin();
		iterator end();
		const_iterator begin() const;
		const_iterator end() const;

		std::string prefix;
		Properties properties;
	};


	/** The XMP Tree */
	class XmpTree
	{
	public:
		typedef std::map<std::string, XmpSchema> DataStore;
		typedef DataStore::iterator iterator;
		typedef DataStore::const_iterator const_iterator;
		typedef DataStore::value_type value_type;
		
		XmpTree();
		virtual ~XmpTree();
		
		bool parse(XmpLoader * loader);

		iterator begin();
		iterator end();
		const_iterator begin() const;
		const_iterator end() const;

	private:
		struct Private;
		Private *d;
	};


}


#endif

