/*
 * exempi - xmpxmlparser.cpp
 *
 * Copyright (C) 2007 Hubert Figuiere
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
 * USA
 */


#ifndef __XMPXMLPARSER_H_
#define __XMPXMLPARSER_H_

#include <stack>

#include <libxml/xmlreader.h>

#include "xmploader.h"
#include "xmptree.h"

namespace Exempi {
	namespace Internal {


		/** @ brief the XML parser for XMP */
		class XmpXmlParser
			: public XmpLoader
		{
		public:
			/** create the XML parse using a buffer */
			XmpXmlParser(const char * buffer, size_t len);
			virtual ~XmpXmlParser();

			/** parse to the store */
			virtual bool parse(XmpTree::DataStore & store);

		private:
			enum {
				CONTEXT_NONE = 0,
				CONTEXT_XMPMETA,
				CONTEXT_RDF,
				CONTEXT_RDFDESCRIPTION,
				CONTEXT_PROPERTY,
				_CONTEXT_LAST
			};
			bool processNode(XmpTree::DataStore & store);
			bool processProperty(XmpSchema::Properties::iterator & prop);
			bool parseBag(XmpValue::Bag &);
			bool parseSeq(XmpValue::Seq &);
			bool parseAlt(XmpValue::Alt &);
			template <typename T>
			bool parseArray(T & array);

			bool parseStructure(XmpValue::Structure &);
			/** check the XML context
			 * @param type the type of node
			 * @param ctx the context requested
			 */
			bool checkContext(int type, int ctx);
			/** XmlTextReader */
			xmlTextReaderPtr m_reader;
			int m_depth;
			std::stack<int> m_context;
			XmpTree::DataStore::iterator m_currentSchema;
			XmpSchema::Properties::iterator m_currentProperty;
		};

	}
}


#endif
